;program: Cubeline
;system : KolibriOS/MenuetOS
;author : Maciej Guba aka macgub
;mail   : macgub3@wp.pl
;web    : www.macgub.hekko.pl
Kolibri = 0
Menuet = 1
OS = Menuet

SIZE_X equ 512
SIZE_Y equ 512
TIMEOUT equ 4
ROUND equ 8
y_offset equ 200
x_offset equ 0
points_count equ 64
lines_count equ 66

use32

	org    0x0

	db     'MENUET01'	; 8 byte id
	dd     0x01		; header version
	dd     START		; start of code
	dd     I_END		; size of image
	dd     I_END		; memory for app
	dd     I_END		; esp
	dd     0x0 , 0x0	; I_Param , I_Icon

START:	  ; start of execution

     call draw_window
     call add_offset
     call scale_points
still:

    mov  eax,23   ; wait here for event with timeout
    mov  ebx,TIMEOUT
  ;  mov eax,11   ; uncomment to full speed mode
    int  0x40

    cmp  eax,1	 ; redraw request ?
    je	red
    cmp  eax,2	 ; key in buffer ?
    je	key
    cmp  eax,3	 ; button in buffer ?
    je	button

    jmp  noclose

  red:	  ; redraw
    call draw_window
    jmp  noclose

  key:	  ; key
    mov  eax,2	 ; just read it and ignore
    int  0x40
    jmp  noclose

  button:   ; button
    mov  eax,17   ; get id
    int  0x40

    cmp  ah,1	; button id=1 ?
    jne  noclose

    mov  eax,-1   ; close this program
    int  0x40
  noclose:

  call calculate_angle	 ; calculates sinus and cosinus
  call copy_points
  call rotate_points
  call translate_points  ; translate from 3d to 2d
  call clear		; clear the screen and intialize Zbuffer
  call draw_lines    ; draw all triangles from the list


  mov eax,7	     ; put image
  mov ebx,screen
  mov ecx,SIZE_X shl 16 + SIZE_Y
if OS = Menuet
  mov edx,5 shl 16 + 20
else
  xor edx,edx
end if
  int 0x40

  jmp  still

include 'grd_line.inc'
scale_points:
  mov ebx,points
  mov ecx,points_count
  fninit
@@:
  fild word[ebx]
  fmul [scale]
  fistp word[ebx]
  fild word[ebx+2]
  fmul [scale]
  fistp word[ebx+2]
  fild word[ebx+4]
  fmul [scale]
  fistp word[ebx+4]
  add ebx,6
  loop @b
ret
add_offset:
  mov ebx,points
  mov ecx,points_count
 @@:
  add word[ebx],x_offset
  add word[ebx+2],y_offset
  add ebx,6
  loop @b
ret
translate_points:
  fninit
  mov ebx,rotated_points
  mov ecx,points_count
 @@:
  fild word[ebx+4] ;z1
  fmul [sq]
  fld st
  fiadd word[ebx]  ;x1
  fistp word[ebx]
  fchs
  fiadd word[ebx+2] ;y1
  fistp word[ebx+2] ;y1

  add ebx,6
  loop @b
ret


calculate_angle:
  finit
  fldpi
  fidiv [i180]
  fimul [angle_counter]
  fsincos
  fstp	[sinbeta]
  fstp	[cosbeta]
  inc	[angle_counter]
  cmp	[angle_counter],360
  jne @f
  mov	[angle_counter],0
 @@:
ret
copy_points:
  mov esi,points
  mov edi,rotated_points
  mov ecx,(points_count*3+2)/2
  cld
  rep movsd
ret
clear:
  cld
  mov  edi,screen
  xor  eax,eax
  mov  ecx,SIZE_X * SIZE_Y * 4 / 3
  rep  stosd
  mov  edi,Zbuffer	     ; make background in z buffer
  mov  eax,-5000*65536+0
  mov  ecx,SIZE_X * SIZE_Y
  rep  stosd
ret
rotate_points:
  fninit
  mov	ebx,rotated_points
  mov	ecx,points_count
 @@:
  mov	ax,word[ebx] ;x
  sub	ax,[xo]
  mov	[xsub],ax
  mov	ax,word[ebx+4] ;z
  sub	ax,[zo]
  mov	[zsub],ax
  fld	[sinbeta]
  fimul [zsub]
  fchs
  fld	[cosbeta]
  fimul [xsub]
  faddp
  fiadd [xo]
  fistp word[ebx]  ;x
  fld	[sinbeta]
  fimul [xsub]
  ;fchs
  fld	[cosbeta]
  fimul [zsub]
  faddp
  fiadd [zo]
  fistp word[ebx+4] ;z

  mov ax,word[ebx+2] ;y ; z axle rotate
  sub ax,[yo]
  mov [ysub],ax
  mov ax,word[ebx]  ;x
  sub ax,[xo]
  mov [xsub],ax
  fld [sinbeta]
  fimul[ysub]
  fld [cosbeta]
  fimul [xsub]
  faddp
  fiadd [xo]
  fistp word[ebx] ;x
  fld [cosbeta]
  fimul [ysub]
  fld [sinbeta]
  fimul [xsub]
  fchs
  faddp
  fiadd [yo]
  fistp word[ebx+2] ;y

  add ebx,6
  dec ecx
  jnz @b
ret

draw_lines:
  mov	ebx,lines
  mov	ecx,lines_count
  mov	esi,Zbuffer
  mov	edi,screen
.again:
  pushad
  movzx edx,word[ebx]
  lea	edx,[edx*3]
  add	edx,edx
  lea	eax,[edx+points]
  fninit
  fild	word[eax+4]
  fmul	[rec_scale]
  fistp word[esp-2]
  movzx ecx,word[esp-2]
  xor	ch,ch
  push	cx
  movzx ecx,word[eax+2]
  sub	cx,550
  push	bx
  mov	bx,cx
  shr	bx,2
  add	cx,bx
  pop	bx
  xor	ch,ch
  push	cx
  movzx ecx,word[eax]
  sub	cx,150
  push	bx
  mov	bx,cx
  shr	bx,2
  add	cx,bx
  pop	bx
  xor	ch,ch
  push	cx

  lea	edx,[edx+rotated_points]
  push	word[edx+4]   ; z coord
  push	word[edx+2]
  push	word[edx]

  movzx edx,word[ebx+2]
  lea	edx,[edx*3]
  add	edx,edx
  lea	eax,[edx+points]
  fninit
  fild	word[eax+4]
  fmul	[rec_scale]
  fistp word[esp-2]
  movzx ecx,word[esp-2]
  xor	ch,ch
  push	cx
  movzx ecx,word[eax+2]
  sub	cx,550
  push	bx
  mov	bx,cx
  shr	bx,2
  add	cx,bx
  pop	bx
  xor	ch,ch
  push	cx
  movzx ecx,word[eax]
  sub	cx,150
  push	bx
  mov	bx,cx
  shr	bx,2
  add	cx,bx
  pop	bx
  xor	ch,ch
  push	cx

  lea	edx,[edx+rotated_points]
  push	word[edx+4]
  push	word[edx+2]
  push	word[edx]

  call	smooth_line
  popad
  add	ebx,4
  dec	ecx
  jnz	.again
ret

;   *********************************************
;   *******  WINDOW DEFINITIONS AND DRAW ********
;   *********************************************

draw_window:

    mov  eax,12 		   ; function 12:tell os about windowdraw
    mov  ebx,1			   ; 1, start of draw
    int  0x40
								   ; DRAW WINDOW
    mov  eax,0	; function 0 : define and draw window
  if OS = Kolibri
    mov  ebx,100*65536+SIZE_X+9      ; [x start] *65536 + [x size]
    mov  ecx,100*65536+SIZE_Y+25     ; [y start] *65536 + [y size]
  else
    mov  ebx,100*65536+SIZE_X+18      ; [x start] *65536 + [x size]
    mov  ecx,100*65536+SIZE_Y+34     ; [y start] *65536 + [y size]
  end if
    mov  edx,0x74000000 		   ; color of work area RRGGBB,8->color gl
    mov  edi,labelt
    int  0x40

    ; WINDOW LABEL
    mov     eax,4	    ; function 4 : write text to window
    mov     ebx,8*65536+8   ; [x start] *65536 + [y start]
    mov     ecx,0x20ddeeff  ; font 1 & color ( 0xF0RRGGBB )
    mov     edx,labelt	    ; pointer to text beginning
    mov     esi,labelen-labelt	   ; text length
    int     0x40

				   ; WINDOW LABEL
    mov  eax,12 		   ; function 12:tell os about windowdraw
    mov  ebx,2			   ; 2, end of draw
    int  0x40

    ret

labelt:
     db   'Cubeline',0
labelen:
sq dd 0.707
scale dd 0.7
rec_scale dd 1.4285
i180 dw 180
angle_counter dw 0
xo dw  SIZE_X/2
yo dw  SIZE_Y/2
zo dw 50
points: ; x, y, z
 dw 150,150,0,	175,150,0, 200,150,0,  225,150,0, 250,150,0, 275,150,0
 dw 300,150,0,	325,150,0, 350,150,0,  350,175,0, 350,200,0, 350,225,0
 dw 350,250,0,	350,275,0, 350,300,0,  350,325,0, 350,350,0, 325,350,0
 dw 300,350,0,	275,350,0, 250,350,0,  225,350,0, 200,350,0, 175,350,0
 dw 150,350,0,	150,325,0, 150,300,0,  150,275,0, 150,250,0, 150,225,0
 dw 150,200,0,	150,175,0

 dw 150,150,255,  175,150,255, 200,150,255,  225,150,255, 250,150,255, 275,150,255
 dw 300,150,255,  325,150,255, 350,150,255,  350,175,255, 350,200,255, 350,225,255
 dw 350,250,255,  350,275,255, 350,300,255,  350,325,255, 350,350,255, 325,350,255
 dw 300,350,255,  275,350,255, 250,350,255,  225,350,255, 200,350,255, 175,350,255
 dw 150,350,255,  150,325,255, 150,300,255,  150,275,255, 150,250,255, 150,225,255
 dw 150,200,255,  150,175,255

lines: ; each line two word - indexes to point list
 dw 0,24,  1,23,  2,22,  3,21,	4,20,  5,19,  6,18,  7,17,  8,16,  0,8,   31,9
 dw 30,10, 29,11, 28,12, 27,13, 26,14, 25,15, 24,16, 0,32,  1,33,  2,34,  3,35
 dw 4,36,  5,37,  6,38,  7,39,	8,40,  9,41,  10,42, 11,43, 12,44, 13,45, 14,46
 dw 15,47, 16,48, 17,49, 18,50, 19,51, 20,52, 21,53, 22,54, 23,55, 24,56, 25,57
 dw 26,58, 27,59, 28,60, 29,61, 30,62, 31,63
 dw 32,56, 33,55, 34,54, 35,53, 36,52, 37,51, 38,50, 39,49, 40,48, 41,63, 42,62
 dw 43,61, 44,60, 45,59, 46,58, 47,57, 48,56
xsub dw ?
zsub dw ?
ysub dw ?
sinbeta dd ?
cosbeta dd ?
rotated_points:
  rw points_count * 3
screen rb SIZE_X * SIZE_Y * 3
Zbuffer rb SIZE_X * SIZE_Y * 4

memStack:
  rw 1024
I_END:
